package com.acemondstudios.rootine.Adapters

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Day
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.UI.Fragments.DayListFragment
import com.acemondstudios.rootine.UI.Fragments.ExerciseListFragment
import com.acemondstudios.rootine.Utils.Utils


class DaysAdapter(
        private val context: Context,
        private val myDataset: List<Day>,
        private val dayListFragment: DayListFragment,
        private val containerActivity: ContainerActivity
) : RecyclerView.Adapter<DaysAdapter.MyViewHolder>() {

    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder.
    // Each data item is just a string in this case that is shown in a TextView.

    class MyViewHolder(context: Context, dayListFragment: DayListFragment, itemView: View) : RecyclerView.ViewHolder(itemView){
        fun bindItems(context: Context, dayListFragment: DayListFragment, element: Day, containerActivity: ContainerActivity){
            val description: TextView = itemView.findViewById(R.id.dayTitle)
            description.text = element.description
            val dayEdit: ImageView = itemView.findViewById(R.id.dayEdit)
            dayEdit.setOnClickListener{
                showDayEditDialog(context, dayListFragment, element)
            }
            itemView.setOnLongClickListener{
                showDayDeleteDialog(context, dayListFragment, element)
                true
            }
            itemView.setOnClickListener{
                val fragment = ExerciseListFragment()
                val args = Bundle()
                args.putSerializable("DAY", element)

                fragment.arguments = args
                containerActivity.openFragment(fragment, true, "ExerciseListFragment")
            }
            val dayView: RelativeLayout = itemView.findViewById(R.id.dayView)
            if(element.done){
                dayView.setBackgroundColor(Color.parseColor("#455A64"))
            }
            else {
                dayView.setBackgroundColor(android.R.attr.selectableItemBackground)
            }
        }

        private fun showDayEditDialog(context: Context, dayListFragment: DayListFragment, day: Day){
            val layoutInflater = LayoutInflater.from(context)
            val promptView: View =
                    layoutInflater.inflate(R.layout.generic_edit_input_dialog, null)
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)
            alertDialogBuilder.setView(promptView)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.accept, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.edit_day))

            val name: EditText = promptView.findViewById(R.id.inputName)

            name.setText(day.description)
            name.requestFocus()
            Utils.showKeyboard(context)

            dialog.setOnShowListener{
                val button = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                button.setOnClickListener{

                    if(name.text.isNotEmpty()) {
                        val db = DBEngine(context)
                        db.editDay(name.text.toString(), day.day_id)
                        dayListFragment.getDaysFromDatabase()
                        dialog.dismiss()
                    }
                    else{
                        name.error = it.context.getString(R.string.required_field)
                    }
                }
            }

            dialog.setOnDismissListener{
                Utils.closeKeyboard(context)
            }

            dialog.show()
        }

        private fun showDayDeleteDialog(context: Context, dayListFragment: DayListFragment, day: Day){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_day))
            dialog.setMessage(context!!.getString(R.string.delete_day_prompt) + " \"" + day.description + "\"?")

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    if(db.dayHasRegisters(day)){
                        // There are registers of this day's exercises. User must accept it before deleting
                        this.showDayWithRegistersDeleteDialog(context, dayListFragment, day)
                        dialog.dismiss()
                    }
                    else {
                        db.deleteDay(day.day_id.toInt())
                        dayListFragment.getDaysFromDatabase()
                        dialog.dismiss()
                    }
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }

            dialog.show()
        }

        private fun showDayWithRegistersDeleteDialog(context: Context, dayListFragment: DayListFragment, day: Day){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_day))
            dialog.setMessage(context!!.getString(R.string.delete_day_register_prompt))

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    db.deleteDay(day.day_id)
                    dayListFragment.getDaysFromDatabase()
                    dialog.dismiss()
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }

            dialog.show()
        }
    }

    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): DaysAdapter.MyViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context).inflate(R.layout.day_recycler_view_item, parent, false)
        return MyViewHolder(context, dayListFragment, layoutInflater)
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        holder.bindItems(context, dayListFragment, myDataset[position], containerActivity)
    }

    // Return the size of your dataset (invoked by the layout manager)
    override fun getItemCount() = myDataset.size
}
