package com.acemondstudios.rootine.Adapters

import android.content.Context
import android.graphics.BitmapFactory
import android.os.Bundle
import android.util.Base64
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Exercise
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.UI.Fragments.ExerciseFragment
import com.acemondstudios.rootine.UI.Fragments.ExerciseListFragment


class ExercicesAdapter(
    private val context: Context,
    private val myDataset: ArrayList<Exercise>,
    private val exerciseListFragment: ExerciseListFragment,
    private val containerActivity: ContainerActivity,
    private val dbEngine: DBEngine
) : RecyclerView.Adapter<ExercicesAdapter.MyViewHolder>() {

    class MyViewHolder(context: Context, exerciseListFragment: ExerciseListFragment, itemView: View, dbEngine: DBEngine) : RecyclerView.ViewHolder(itemView){

        fun bindItems(context: Context, exerciseListFragment: ExerciseListFragment, element: Exercise, showHeader: Boolean, containerActivity: ContainerActivity, dbEngine: DBEngine){
            val exerciseTitle: TextView = itemView.findViewById(R.id.exerciseTitle)
            val exerciseSeries: TextView = itemView.findViewById(R.id.exerciseSeries)
            val exerciseImage: ImageView = itemView.findViewById(R.id.exerciseImage)
            val categoryName: TextView = itemView.findViewById(R.id.categoryName)

            if(showHeader){
                categoryName.text = element.category.description
                categoryName.visibility = View.VISIBLE
            }
            else{
                categoryName.visibility = View.GONE
            }
            exerciseTitle.text = element.title

            exerciseSeries.text = element.series.toString() + " x " + element.min

            if(element.min != element.max){
                exerciseSeries.text = exerciseSeries.text as String + "-" + element.max
            }

            if(element.uid.isNotEmpty()){
                val base64 = dbEngine.readImage(element.uid)
                if(base64.isNotEmpty()){
                    val decodedString: ByteArray = Base64.decode(base64, Base64.DEFAULT)
                    val decodedByte = BitmapFactory.decodeByteArray(decodedString, 0, decodedString.size)
                    exerciseImage.setImageBitmap(decodedByte)
                    exerciseImage.scaleType = ImageView.ScaleType.CENTER_CROP
                }
            }

            itemView.setOnClickListener{
                val fragment = ExerciseFragment()
                val args = Bundle()
                args.putSerializable("EXERCISE", element)

                fragment.arguments = args
                containerActivity.openFragment(fragment, true, "ExerciseFragment")
            }

            itemView.setOnLongClickListener{
                showExerciseDeleteDialog(context, exerciseListFragment, element)
                true
            }
        }

        private fun showExerciseDeleteDialog(context: Context, exerciseListFragment: ExerciseListFragment, exercise: Exercise){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_exercise))
            dialog.setMessage(context!!.getString(R.string.delete_exercise_prompt) + " \"" + exercise.title + "\"?")

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    if(db.exerciseHasRegisters(exercise)){
                        // There are registers of this exercise. User must accept it before deleting
                        this.showExerciseWithRegistersDeleteDialog(context, exerciseListFragment, exercise)
                        dialog.dismiss()
                    }
                    else {
                        db.deleteExercise(exercise)
                        exerciseListFragment.getExercisesFromDatabase()
                        dialog.dismiss()
                    }
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }

            dialog.show()
        }

        private fun showExerciseWithRegistersDeleteDialog(context: Context, exerciseListFragment: ExerciseListFragment, exercise: Exercise){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_exercise))
            dialog.setMessage(context!!.getString(R.string.delete_exercise_register_prompt))

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    db.deleteExercise(exercise)
                    exerciseListFragment.getExercisesFromDatabase()
                    dialog.dismiss()
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }
            dialog.show()
        }
    }

    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): ExercicesAdapter.MyViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context).inflate(R.layout.exercise_recycler_view_item, parent, false)
        return MyViewHolder(context, exerciseListFragment, layoutInflater, dbEngine)
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element

        // Primer cal saber si aquest exercici té una categoria diferent de l'anterior, per així
        // mostrar o no la capçalera amb el títol d'aquesta.

        var showHeader = true
        if(position > 0) {
            val currentCategoryID = myDataset[position].category.category_id
            val previousCategoryID = myDataset[position-1].category.category_id
            if(currentCategoryID == previousCategoryID)
                showHeader = false
        }


        holder.bindItems(context, exerciseListFragment, myDataset[position], showHeader, containerActivity, dbEngine)
    }

    // Return the size of your dataset (invoked by the layout manager)
    override fun getItemCount() = myDataset.size
}
