package com.acemondstudios.rootine.Adapters

import android.content.Context
import android.graphics.BitmapFactory
import android.os.Bundle
import android.util.Base64
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Exercise
import com.acemondstudios.rootine.Models.RoutineHistoryRegister
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.UI.Fragments.ExerciseFragment
import com.acemondstudios.rootine.UI.Fragments.ExerciseListFragment
import com.acemondstudios.rootine.UI.Fragments.UserFragment
import com.acemondstudios.rootine.Utils.Utils


class RoutineHistoryRegisterAdapter(
    private val context: Context,
    private val myDataset: ArrayList<RoutineHistoryRegister>,
    private val userFragment: UserFragment,
    private val dbEngine: DBEngine
) : RecyclerView.Adapter<RoutineHistoryRegisterAdapter.MyViewHolder>() {

    class MyViewHolder(context: Context, itemView: View, userFragment: UserFragment, dbEngine: DBEngine) : RecyclerView.ViewHolder(itemView){

        fun bindItems(context: Context, element: RoutineHistoryRegister, showHeader: Boolean, userFragment: UserFragment, dbEngine: DBEngine){
            val dayTitle: TextView = itemView.findViewById(R.id.dayTitle)
            val exerciseDate: TextView = itemView.findViewById(R.id.exerciseDate)
            val routineTitle: TextView = itemView.findViewById(R.id.routineTitle)

            if(showHeader){
                routineTitle.text = element.routine_description
                routineTitle.visibility = View.VISIBLE
            }
            else{
                routineTitle.visibility = View.GONE
            }
            dayTitle.text = element.day_description

            exerciseDate.text = Utils.getFormattedDate(element.date_done)

            itemView.setOnLongClickListener{
                showRoutineHistoryRegisterDeleteDialog(context, userFragment, element)
                true
            }
        }

        private fun showRoutineHistoryRegisterDeleteDialog(context: Context, userFragment: UserFragment, routineHistoryRegister: RoutineHistoryRegister){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_routine_history_register_title))
            dialog.setMessage(context!!.getString(R.string.delete_routine_history_register_prompt) + " " + Utils.getFormattedDate(routineHistoryRegister.date_done) + "?")

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    db.deleteRoutineHistoryRegister(routineHistoryRegister)
                    userFragment.getRoutineHistoryRegistersFromDatabase()
                    dialog.dismiss()
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }
            dialog.show()
        }
    }

    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): RoutineHistoryRegisterAdapter.MyViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context).inflate(R.layout.routine_history_register_recycler_view_item, parent, false)
        return MyViewHolder(context, layoutInflater, userFragment, dbEngine)
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element

        // Primer cal saber si aquest exercici té una categoria diferent de l'anterior, per així
        // mostrar o no la capçalera amb el títol d'aquesta.

        var showHeader = true
        if(position > 0) {
            val currentRoutineID = myDataset[position].routine_id
            val previousRoutineID = myDataset[position-1].routine_id
            if(currentRoutineID == previousRoutineID)
                showHeader = false
        }


        holder.bindItems(context, myDataset[position], showHeader, userFragment, dbEngine)
    }

    // Return the size of your dataset (invoked by the layout manager)
    override fun getItemCount() = myDataset.size
}
