package com.acemondstudios.rootine.Adapters

import android.content.Context
import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Routine
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.UI.Fragments.DayListFragment
import com.acemondstudios.rootine.UI.Fragments.RoutineListFragment
import com.acemondstudios.rootine.Utils.Utils


class RoutinesAdapter(
    private val context: Context,
    private val myDataset: List<Routine>,
    private val routineListFragment: RoutineListFragment
) : RecyclerView.Adapter<RoutinesAdapter.MyViewHolder>() {

    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder.
    // Each data item is just a string in this case that is shown in a TextView.

    class MyViewHolder(context: Context, routineListFragment: RoutineListFragment, itemView: View) : RecyclerView.ViewHolder(itemView){
        fun bindItems(context: Context, routineListFragment: RoutineListFragment, element: Routine){
            val description: TextView = itemView.findViewById(R.id.routineTitle)
            val date: TextView = itemView.findViewById(R.id.routineDate)
            description.text = element.description
            date.text = context.resources.getString(R.string.start_date) + ": " + Utils.getSimpleFormattedDate(element.create_date)
            val routineEdit: ImageView = itemView.findViewById(R.id.routineEdit)
            routineEdit.setOnClickListener{
                showRoutineEditDialog(context, routineListFragment, element)
            }
            itemView.setOnLongClickListener{
                showRoutineDeleteDialog(context, routineListFragment, element)
                true
            }
            itemView.setOnClickListener{
                val intent = Intent(context, ContainerActivity::class.java)
                val fragment = DayListFragment()

                intent.putExtra("FRAGMENT", fragment)
                intent.putExtra("ROUTINE", element)
                intent.putExtra("TAG", "DayListFragment")
                context.startActivity(intent)
            }
        }

        private fun showRoutineEditDialog(context: Context, routineListFragment: RoutineListFragment, routine: Routine){
            val layoutInflater = LayoutInflater.from(context)
            val promptView: View =
                layoutInflater.inflate(R.layout.generic_edit_input_dialog, null)
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)
            alertDialogBuilder.setView(promptView)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.accept, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.edit_routine))

            val name: EditText = promptView.findViewById(R.id.inputName)

            name.setText(routine.description)
            name.requestFocus()
            Utils.showKeyboard(context)

            dialog.setOnShowListener{
                val button = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                button.setOnClickListener{

                    if(name.text.isNotEmpty()) {
                        val db = DBEngine(context)
                        db.editRoutine(name.text.toString(), routine.routine_id)
                        routineListFragment.getRoutinesFromDatabase()
                        dialog.dismiss()
                    }
                    else{
                        name.error = it.context.getString(R.string.required_field)
                    }
                }
            }

            dialog.setOnDismissListener{
                Utils.closeKeyboard(context)
            }

            dialog.show()
        }

        private fun showRoutineDeleteDialog(context: Context, routineListFragment: RoutineListFragment, routine: Routine){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_routine))
            dialog.setMessage(context!!.getString(R.string.delete_prompt) + " \"" + routine.description + "\"?")

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    if(db.routineHasRegisters(routine)){
                        this.showRoutineWithRegistersDeleteDialog(context, routineListFragment, routine)
                        dialog.dismiss()
                    }
                    else{
                        db.deleteRoutine(routine.routine_id)
                        routineListFragment.getRoutinesFromDatabase()
                        dialog.dismiss()
                    }
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }

            dialog.show()
        }

        private fun showRoutineWithRegistersDeleteDialog(context: Context, routineListFragment: RoutineListFragment, routine: Routine){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_routine))
            dialog.setMessage(context!!.getString(R.string.delete_register_prompt))

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    db.deleteRoutine(routine.routine_id.toInt())
                    routineListFragment.getRoutinesFromDatabase()
                    dialog.dismiss()
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }

            dialog.show()
        }
    }

    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): RoutinesAdapter.MyViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context).inflate(R.layout.routine_recycler_view_item, parent, false)
        return MyViewHolder(context, routineListFragment, layoutInflater)
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        holder.bindItems(context, routineListFragment, myDataset[position])
    }

    // Return the size of your dataset (invoked by the layout manager)
    override fun getItemCount() = myDataset.size


}
