package com.acemondstudios.rootine.Adapters

import android.content.Context
import android.text.InputType
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.UserWeight
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.UI.Fragments.WeightHistoryFragment
import com.acemondstudios.rootine.Utils.Utils


class UserWeightAdapter(
    private val context: Context,
    private val myDataset: List<UserWeight>,
    private val weightHistoryFragment: WeightHistoryFragment,
    private val containerActivity: ContainerActivity
) : RecyclerView.Adapter<UserWeightAdapter.MyViewHolder>() {

    // Provide a reference to the views for each data item
    // Complex data items may need more than one view per item, and
    // you provide access to all the views for a data item in a view holder.
    // Each data item is just a string in this case that is shown in a TextView.

    class MyViewHolder(context: Context, weightHistoryFragment: WeightHistoryFragment, itemView: View) : RecyclerView.ViewHolder(itemView){
        fun bindItems(context: Context, weightHistoryFragment: WeightHistoryFragment, element: UserWeight, containerActivity: ContainerActivity){
            val weightRecord: TextView = itemView.findViewById(R.id.weightRecord)
            weightRecord.text = element.weight.toString().replace('.', ',') + " kg"

            val weightRecordDate: TextView = itemView.findViewById(R.id.weightDate)
            weightRecordDate.text = Utils.getSimpleFormattedDate(element.record_date)

            val weightRecordEdit: ImageView = itemView.findViewById(R.id.weightEdit)
            weightRecordEdit.setOnClickListener{
                showUserWeightEditDialog(context, weightHistoryFragment, element)
            }
            itemView.setOnLongClickListener{
                showUserWeightDeleteDialog(context, weightHistoryFragment, element)
                true
            }
            itemView.setOnClickListener{
                showUserWeightEditDialog(context, weightHistoryFragment, element)
            }
        }

        private fun showUserWeightEditDialog(context: Context, weightHistoryFragment: WeightHistoryFragment, userWeight: UserWeight){
            val layoutInflater = LayoutInflater.from(context)
            val promptView: View =
                layoutInflater.inflate(R.layout.generic_edit_input_dialog, null)
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)
            alertDialogBuilder.setView(promptView)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.accept, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.edit_user_weight))

            val name: EditText = promptView.findViewById(R.id.inputName)

            name.inputType = InputType.TYPE_NUMBER_FLAG_DECIMAL
            name.setText(userWeight.weight.toString())
            name.requestFocus()
            Utils.showKeyboard(context)

            dialog.setOnShowListener{
                val button = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                button.setOnClickListener{

                    if(name.text.isNotEmpty()) {
                        val db = DBEngine(context)
                        db.updateUserWeight(Utils.getStringDate(userWeight.record_date), userWeight.weight, name.text.toString().toDouble())
                        weightHistoryFragment.getUserWeightsFromDatabase()
                        dialog.dismiss()
                    }
                    else{
                        name.error = it.context.getString(R.string.required_field)
                    }
                }
            }

            dialog.setOnDismissListener{
                Utils.closeKeyboard(context)
            }

            dialog.show()
        }

        private fun showUserWeightDeleteDialog(context: Context, weightHistoryFragment: WeightHistoryFragment, userWeight: UserWeight){
            val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)

            // setup a dialog window
            alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
            alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

            val dialog = alertDialogBuilder.create()
            dialog.setTitle(context!!.getString(R.string.delete_user_weight))
            dialog.setMessage(context!!.getString(R.string.delete_user_weight) + " \"" + userWeight.weight + " kg - " + Utils.getStringDate(userWeight.record_date) + "\"?")

            dialog.setOnShowListener{
                val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                buttonYes.setOnClickListener{
                    val db = DBEngine(context)
                    db.deleteUserWeight(Utils.getStringDate(userWeight.record_date), userWeight.weight)
                    weightHistoryFragment.getUserWeightsFromDatabase()
                    dialog.dismiss()
                }

                val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
                buttonNo.setOnClickListener{
                    dialog.dismiss()
                }
            }

            dialog.show()
        }
    }

    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): UserWeightAdapter.MyViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context).inflate(R.layout.weight_recycler_view_item, parent, false)
        return MyViewHolder(context, weightHistoryFragment, layoutInflater)
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        // - get element from your dataset at this position
        // - replace the contents of the view with that element
        holder.bindItems(context, weightHistoryFragment, myDataset[position], containerActivity)
    }

    // Return the size of your dataset (invoked by the layout manager)
    override fun getItemCount() = myDataset.size
}
