package com.acemondstudios.rootine.UI.Activities

import android.content.Intent
import android.content.SharedPreferences
import android.graphics.BitmapFactory
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.util.Base64
import android.util.Base64.decode
import android.view.MenuItem
import android.view.View
import android.widget.Button
import android.widget.FrameLayout
import android.widget.ImageView
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.FileProvider
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentTransaction
import com.acemondstudios.rootine.BuildConfig
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Exercise
import com.acemondstudios.rootine.Models.Register
import com.acemondstudios.rootine.Models.Routine
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Fragments.RoutineWizardFragment
import java.io.File
import java.io.IOException
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList


class ContainerActivity : AppCompatActivity() {
    val REQUEST_TAKE_PHOTO = 1

    var currentPhotoPath: String = ""
    private var mExercises: ArrayList<Exercise> = ArrayList()
    private var mCurrentExercise: Int = 0
    private var mRegisters: ArrayList<Register> = ArrayList()
    lateinit var mSharedPreferences: SharedPreferences
    private lateinit var mImageView: ImageView

    private val mFrameLayout: FrameLayout by lazy {
        findViewById(R.id.frameLayout)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_container)

        mSharedPreferences = getPreferences(MODE_PRIVATE)
        if(mSharedPreferences.contains("currentPhotoPath")){
            currentPhotoPath = mSharedPreferences.getString("currentPhotoPath", "").toString()
        }

        // If things get messy...
        //val db = DBHelper(this)
        //db.deleteAllImages()

        if(intent.extras != null) {
            val extras: Bundle = intent.extras as Bundle
            val fragment = extras.get("FRAGMENT") as Fragment
            var tag = extras.get("TAG") as String
            if(extras.containsKey("ROUTINE")){
                val arguments = Bundle()
                arguments.putSerializable("ROUTINE", extras.get("ROUTINE") as Routine)
                fragment.arguments = arguments
            }
            else if(extras.containsKey("EXERCISES")){
                mExercises = extras.get("EXERCISES") as ArrayList<Exercise>
                if(mExercises.count() > 0) {
                    val arguments = Bundle()
                    arguments.putSerializable("CURRENT_EXERCISE", mExercises[mCurrentExercise])
                    fragment.arguments = arguments
                    tag = tag + "_" + mCurrentExercise
                    setExerciseActionBarTitle()
                }
            }
//            if(tag.compareTo("WeightHistoryFragment") == 0){
//                openFragment(fragment, true, tag)
//            }
            openFragment(fragment, false, tag)
            mImageView = ImageView(this)
        }


    }

    fun openFragment(fragment: Fragment, empilar: Boolean, tag: String) {
        val fragmentManager: FragmentManager = supportFragmentManager
        if(fragmentManager.findFragmentByTag(tag) == null) {
            val fragmentTransaction: FragmentTransaction = fragmentManager.beginTransaction()
            fragmentTransaction.replace(mFrameLayout.id, fragment, tag)
            fragmentTransaction.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
            fragmentTransaction.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_CLOSE)
            if (empilar) {
                fragmentTransaction.addToBackStack(null) // per afegir-lo a la pila, i que es mantingui al fer back
            }
            fragmentTransaction.commit()
        }
    }

    fun currentRegisterManagement(usedWeight: Double, series: Int, min: Int, max: Int){
        if(mCurrentExercise < mRegisters.count()){
            var currentRegister = mRegisters[mCurrentExercise]
            currentRegister.weight = usedWeight
            currentRegister.series = series
            currentRegister.min = min
            currentRegister.max = max
        }
        else {
            // New register
            val currentExercise = mExercises[mCurrentExercise]
            val newRegister = Register(
                currentExercise.exercise_id,
                Calendar.getInstance(),
                currentExercise.day_id,
                currentExercise.routine_id,
                series,
                min,
                max,
                usedWeight
            )
            mRegisters.add(newRegister)
        }
    }

    fun setImageView(imageView: ImageView){
        mImageView = imageView
    }

    fun nextButton(usedWeight: Double, series: Int, min: Int, max: Int){
        currentRegisterManagement(usedWeight, series, min, max)
        mCurrentExercise++
        openExerciseFragment()
    }

    fun previousButton(usedWeight: Double, series: Int, min: Int, max: Int){
        currentRegisterManagement(usedWeight, series, min, max)
        mCurrentExercise--
        openExerciseFragment()
    }

    fun finishButton(usedWeight: Double, series: Int, min: Int, max: Int){
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(this)
        currentRegisterManagement(usedWeight, series, min, max)
        mCurrentExercise++
        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
        alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

        val dialog = alertDialogBuilder.create()
        dialog.setTitle(getString(R.string.finish_exercise_title))
        dialog.setMessage(getString(R.string.finish_routine_prompt))

        dialog.setOnShowListener{
            val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
            buttonYes.setOnClickListener{
                if(mRegisters.count() > 0) {
                    var db = DBEngine(this)
                    db.saveRegisters(mRegisters)
                    if(db.allDaysDone(mRegisters[0].routine_id)){
                        // All days done
                        db.resetRoutineDays(mRegisters[0].routine_id)
                    }
                    else {
                        // Mark day as done
                        db.markDayAsDone(mRegisters[0].day_id)
                    }
                }
                dialog.dismiss()
                super.onBackPressed()
            }

            val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
            buttonNo.setOnClickListener{
                dialog.dismiss()
            }
        }
        dialog.show()
    }

    fun cancelWizard() {
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(this)

        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
        alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

        val dialog = alertDialogBuilder.create()
        dialog.setTitle(getString(R.string.cancel_exercise_title))
        dialog.setMessage(getString(R.string.cancel_routine_prompt))

        dialog.setOnShowListener{
            val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
            buttonYes.setOnClickListener{
                dialog.dismiss()
                super.onBackPressed()
            }

            val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
            buttonNo.setOnClickListener{
                dialog.dismiss()
            }
        }
        dialog.show()
    }

    fun openExerciseFragment(){
        //if(mCurrentExercise >= 0 && mCurrentExercise < mExercises.count()) {
        val routineWizardFragment = RoutineWizardFragment()
        val arguments = Bundle()
        arguments.putSerializable("CURRENT_EXERCISE", mExercises[mCurrentExercise])

        if(mCurrentExercise < mRegisters.count()){
            arguments.putSerializable("CURRENT_REGISTER", mRegisters[mCurrentExercise])
        }

        routineWizardFragment.arguments = arguments
        openFragment(routineWizardFragment, false, "RoutineWizardFragment_$mCurrentExercise")
        setExerciseActionBarTitle()
        //}
    }

    fun isLastExercise(): Boolean {
        return mCurrentExercise == (mExercises.count() - 1)
    }

    fun isFirstExercise(): Boolean {
        return mCurrentExercise == 0
    }

    private fun setExerciseActionBarTitle(){
        val screenTitle = getString(R.string.app_name) + ": " + getString(R.string.exercise) + " " + (mCurrentExercise+1) + " " + getString(
            R.string.of
        ) + " " + mExercises.count()
        setActionBarTitle(screenTitle)
    }

    fun setActionBarTitle(title: String?) {
        supportActionBar!!.title = title
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        if(mExercises.count() == 0){
            super.onBackPressed()
        }
    }
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        fillImage()
    }

    fun fillImage(){
        if(currentPhotoPath.isNotEmpty()){
            val imgFile = File(currentPhotoPath)

            if (imgFile.exists()) {
                val myBitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
                if(myBitmap != null) {
                    mImageView.visibility = View.VISIBLE
                    mImageView.setImageBitmap(myBitmap)
                }
            }
        }
    }

    fun fillImageFromBase64(base64: String){
        val decodedString: ByteArray = decode(base64, Base64.DEFAULT)
        val decodedByte = BitmapFactory.decodeByteArray(decodedString, 0, decodedString.size)
        if(decodedByte != null) {
            mImageView.visibility = View.VISIBLE
            mImageView.setImageBitmap(decodedByte)
        }
    }

    fun clearImage(){
        mImageView.setImageBitmap(null)
        currentPhotoPath = ""
        mSharedPreferences.edit().remove("currentPhotoPath").apply()
        mImageView.visibility = View.GONE
    }

    fun imageIsEmpty(): Boolean {
        return mImageView.drawable == null
    }

    @Throws(IOException::class)
    private fun createImageFile(): File {
        // Create an image file name
        val timeStamp: String = SimpleDateFormat("yyyyMMdd_HHmmss").format(Date())
        val storageDir: File? = getExternalFilesDir(Environment.DIRECTORY_PICTURES)
        return File.createTempFile(
            "JPEG_${timeStamp}_", /* prefix */
            ".jpg", /* suffix */
            storageDir /* directory */
        ).apply {
            // Save a file: path for use with ACTION_VIEW intents
            currentPhotoPath = absolutePath
            mSharedPreferences.edit().putString("currentPhotoPath", currentPhotoPath).apply()
        }
    }

    fun dispatchTakePictureIntent() {
        Intent(MediaStore.ACTION_IMAGE_CAPTURE).also { takePictureIntent ->
            // Ensure that there's a camera activity to handle the intent
            takePictureIntent.resolveActivity(packageManager)?.also {
                // Create the File where the photo should go
                val photoFile: File? = try {
                    createImageFile()
                } catch (ex: IOException) {

                    null
                }
                // Continue only if the File was successfully created
                photoFile?.also {
                    val photoURI = FileProvider.getUriForFile(
                        Objects.requireNonNull(this),
                        BuildConfig.APPLICATION_ID + ".provider", it
                    );
                    takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI)
                    startActivityForResult(takePictureIntent, REQUEST_TAKE_PHOTO)
                }
            }
        }
    }
}