package com.acemondstudios.rootine.UI.Fragments


import android.content.Context
import android.os.Bundle
import android.view.*
import android.widget.Button
import androidx.fragment.app.Fragment
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Adapters.DaysAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Day
import com.acemondstudios.rootine.Models.Routine

import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.Utils.Utils
import com.google.android.material.floatingactionbutton.FloatingActionButton
import java.io.Serializable

class DayListFragment : Fragment(), Serializable {

    private lateinit var mRoutine: Routine
    private lateinit var mActivity: ContainerActivity
    private lateinit var recyclerView: RecyclerView
    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: LinearLayoutManager
    private var myDataset: ArrayList<Day> = ArrayList()
    private lateinit var mContext: Context

    private lateinit var no_data: TextView
    private lateinit var list_icon: ImageView

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_day_list, container, false)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        inflater.inflate(R.menu.restart_days_menu, menu)
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.restart_item -> {
            showRestartDialog()
            true
        }

        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun showRestartDialog(){
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(mContext)

        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
        alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

        val dialog = alertDialogBuilder.create()
        dialog.setTitle(mContext.getString(R.string.restart_days_title))
        dialog.setMessage(mContext.getString(R.string.restart_days_prompt))

        dialog.setOnShowListener{
            val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
            buttonYes.setOnClickListener{
                val db = DBEngine(mContext)
                db.resetRoutineDays(mRoutine.routine_id)
                getDaysFromDatabase()
                dialog.dismiss()
            }

            val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
            buttonNo.setOnClickListener{
                dialog.dismiss()
            }
        }

        dialog.show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        mActivity = activity as ContainerActivity
        mContext = context as Context

        if(arguments != null) {
            val bundle = arguments as Bundle
            mRoutine = bundle.get("ROUTINE") as Routine
            mActivity.setActionBarTitle(mRoutine.description)

            viewManager = LinearLayoutManager(context)
            viewAdapter = DaysAdapter(context!!, myDataset, this, mActivity)

            no_data = view.findViewById(R.id.no_data)
            list_icon = view.findViewById(R.id.list_icon)

            recyclerView = view.findViewById(R.id.daysRecyclerView)

            recyclerView.apply {
                // use this setting to improve performance if you know that changes
                // in content do not change the layout size of the RecyclerView
                setHasFixedSize(true)

                // use a linear layout manager
                layoutManager = viewManager

                // specify an viewAdapter (see also next example)
                adapter = viewAdapter
            }

            val dividerItemDecoration = DividerItemDecoration(
                    recyclerView.context,
                    viewManager.orientation
            )
            recyclerView.addItemDecoration(dividerItemDecoration)

            val newDay: FloatingActionButton = view.findViewById(R.id.newDay)

            newDay.setOnClickListener{
                showNewDayDialog(it.context)
            }

            getDaysFromDatabase()
        }
        mActivity.supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }

    fun showNewDayDialog(context: Context){
        val layoutInflater = LayoutInflater.from(context)
        val promptView: View =
                layoutInflater.inflate(R.layout.generic_edit_input_dialog, null)
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)
        alertDialogBuilder.setView(promptView)

        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.accept, null)

        val dialog = alertDialogBuilder.create()
        dialog.setTitle(context!!.getString(R.string.new_day))

        val name: EditText = promptView.findViewById(R.id.inputName)
        name.hint = getString(R.string.day_name)

        name.requestFocus()
        Utils.showKeyboard(context)

        dialog.setOnShowListener{
            val button = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
            button.setOnClickListener{
                if(name.text.isNotEmpty()) {
                    val db = DBEngine(context)
                    db.addDay(mRoutine.routine_id, name.text.toString())
                    dialog.dismiss()
                    getDaysFromDatabase()
                }
                else{
                    name.error = it.context.getString(R.string.required_field)
                }
            }
        }

        dialog.setOnDismissListener{
            Utils.closeKeyboard(context)
        }

        dialog.show()
    }

    fun getDaysFromDatabase(){
        val db = DBEngine(context!!)
        val days: ArrayList<Day> = db.getAllDaysFromRoutine(mRoutine.routine_id.toInt())
        myDataset.clear()
        myDataset.addAll(days)
        viewAdapter.notifyDataSetChanged()
        checkNumberOfItems()
    }

    private fun checkNumberOfItems(){
        if(myDataset.size > 0){
            no_data.visibility = View.GONE
            list_icon.visibility = View.GONE
        }
        else{
            no_data.visibility = View.VISIBLE
            list_icon.visibility = View.VISIBLE
        }
    }
}
