package com.acemondstudios.rootine.UI.Fragments

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.util.Base64
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.Fragment
import com.acemondstudios.rootine.Adapters.CategorySpinnerAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Category
import com.acemondstudios.rootine.Models.Day
import com.acemondstudios.rootine.Models.Exercise
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.Utils.Utils
import com.google.android.material.textfield.TextInputLayout
import java.io.ByteArrayOutputStream
import java.io.File


class ExerciseFragment : Fragment() {
    private lateinit var mSaveButton: Button
    private lateinit var mAddImageButton: Button
    private lateinit var mDeleteImageButton: Button
    private lateinit var mExercise: Exercise
    private lateinit var mContext: Context

    private lateinit var mCategorySpinner: Spinner
    private lateinit var mCategories: ArrayList<Category>
    private lateinit var mNewCategory: EditText
    private lateinit var mNewCategoryLayout: TextInputLayout
    private lateinit var mTitle: EditText
    private lateinit var mDescription: EditText
    private lateinit var mSeries: EditText
    private lateinit var mMin: EditText
    private lateinit var mMax: EditText
    private lateinit var mDay: Day
    private lateinit var mActivity: ContainerActivity
    private lateinit var db: DBEngine
    private var mDeleteImage = false
    private var mIsNew = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = inflater.inflate(R.layout.fragment_exercise, container, false)
        mActivity = activity as ContainerActivity

        mContext = context as Context
        db = DBEngine(mContext)

        if(mActivity.mSharedPreferences.contains("deleteImage")){
            mDeleteImage = mActivity.mSharedPreferences.getBoolean("deleteImage", false)
        }

        mCategories = db.getAllCategories()

        if(arguments != null) {
            val bundle = arguments as Bundle

            if(bundle.containsKey("NEW"))
                mIsNew = true
            if(bundle.containsKey("DAY"))
                mDay = bundle.getSerializable("DAY") as Day

            var hasRegisters = false
            if(bundle.containsKey("EXERCISE")) {
                mExercise = bundle.getSerializable("EXERCISE") as Exercise
                hasRegisters = db.exerciseHasRegisters(mExercise)
            }

            mSaveButton = view.findViewById(R.id.saveChangesButton)
            mCategorySpinner = view.findViewById(R.id.categorySpinner)
            mNewCategory = view.findViewById(R.id.newCategory)
            mTitle = view.findViewById(R.id.title)
            mDescription = view.findViewById(R.id.description)
            mSeries = view.findViewById(R.id.series)
            mMin = view.findViewById(R.id.min)
            mMax = view.findViewById(R.id.max)
            mAddImageButton = view.findViewById(R.id.addImageButton)
            mDeleteImageButton = view.findViewById(R.id.deleteImageButton)
            mNewCategoryLayout = view.findViewById(R.id.newCategoryLayout)

            mCategorySpinner.adapter = CategorySpinnerAdapter(
                mContext,
                android.R.layout.simple_spinner_dropdown_item,
                mCategories
            )

            if(mCategories.count() == 0){
                mCategorySpinner.visibility = View.GONE
            }

            mActivity.supportActionBar?.setDisplayHomeAsUpEnabled(true)

            if(!mIsNew){
                if(hasRegisters){
                    mCategorySpinner.isEnabled = false
                    mTitle.isEnabled = false
                    mDescription.isEnabled = false
                    mSeries.isEnabled = false
                    mMin.isEnabled = false
                    mMax.isEnabled = false
                    mAddImageButton.visibility = View.GONE
                    mDeleteImageButton.visibility = View.GONE
                    mNewCategory.visibility = View.GONE
                    mSaveButton.visibility = View.GONE
                    mNewCategoryLayout.visibility = View.GONE
                }
                var index = 0
                var trobat = false
                while(index < mCategories.count() && !trobat){
                    val current = mCategories[index]
                    if(current.category_id == mExercise.category.category_id){
                        trobat = true
                    }
                    else
                        index++
                }

                mCategorySpinner.setSelection(index)
                mTitle.setText(mExercise.title)
                mDescription.setText(mExercise.description)
                mSeries.setText(mExercise.series.toString())
                mMin.setText(mExercise.min.toString())
                mMax.setText(mExercise.max.toString())

                if(mExercise.uid.isNotEmpty()){
                    // Get, decode and draw image
                    val base64 = db.readImage(mExercise.uid)
                    if(base64.isNotEmpty()) {
                        mActivity.fillImageFromBase64(base64)
                        mDeleteImageButton.visibility = View.VISIBLE
                    }
                }
            }

            mSaveButton.setOnClickListener {
                var categoryID = 1;

                if(mNewCategory.text.isNotEmpty()){
                    // New category to add
                    categoryID = db.addCategory(mNewCategory.text.toString())
                }
                else {
                    val currentCategory: Category = mCategorySpinner.selectedItem as Category
                    categoryID = currentCategory.category_id
                }

                var base64 = ""

                val currentPhotoPath = mActivity.currentPhotoPath

                if(currentPhotoPath.isNotEmpty()){
                    // There's an image
                    val imgFile = File(currentPhotoPath)

                    if (imgFile.exists()) {
                        var myBitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
                        if(myBitmap != null) {
                            myBitmap = Utils.getResizedBitmap(myBitmap, 640)
                            val byteArrayOutputStream = ByteArrayOutputStream()
                            myBitmap.compress(Bitmap.CompressFormat.JPEG, 60, byteArrayOutputStream)
                            val byteArray: ByteArray = byteArrayOutputStream.toByteArray()
                            base64 = Base64.encodeToString(byteArray, Base64.DEFAULT)
                        }
                    }
                }

                if(mDeleteImage) {
                    db.deleteImage(mExercise.uid)
                    mExercise.uid = ""
                    mActivity.mSharedPreferences.edit().remove("deleteImage").apply()
                }

                if(mIsNew){
                    var withErrors = false

                    if(mCategories.count() == 0 && !mNewCategory.text.isNotEmpty()){
                        mNewCategory.error =getString(R.string.required_field)
                        withErrors = true
                    }

                    if(!mTitle.text.isNotEmpty()) {
                        mTitle.error = getString(R.string.required_field)
                        withErrors = true
                    }

                    if(!mDescription.text.isNotEmpty()) {
                        mDescription.error = getString(R.string.required_field)
                        withErrors = true
                    }

                    if(!mSeries.text.isNotEmpty()) {
                        mSeries.error = getString(R.string.required_field)
                        withErrors = true
                    }

                    if(!mMin.text.isNotEmpty()) {
                        mMin.error = getString(R.string.required_field)
                        withErrors = true
                    }

                    if(!mMax.text.isNotEmpty()) {
                        mMax.error = getString(R.string.required_field)
                        withErrors = true
                    }

                    if(!withErrors){
                        // All required fields has been successfully filled. Now it's time to add
                        // the exercise.

                        db.addExercise(
                            Integer.parseInt(mDay.day_id.toString()),
                            Integer.parseInt(mDay.routine_id.toString()),
                            categoryID,
                            mTitle.text.toString(),
                            mDescription.text.toString(),
                            Integer.parseInt(mSeries.text.toString()),
                            Integer.parseInt(mMin.text.toString()),
                            Integer.parseInt(mMax.text.toString()),
                            base64
                        )
                        activity?.onBackPressed()
                    }

                }
                else{
                    // Edit current exercise
                    db.updateExercise(
                        mExercise.exercise_id,
                        mExercise.day_id,
                        mExercise.routine_id,
                        categoryID,
                        mTitle.text.toString(),
                        mDescription.text.toString(),
                        Integer.parseInt(mSeries.text.toString()),
                        Integer.parseInt(mMin.text.toString()),
                        Integer.parseInt(mMax.text.toString()),
                        mExercise.uid,
                        base64
                    )
                    activity?.onBackPressed()
                }
            }

            mAddImageButton.setOnClickListener{
                mActivity.dispatchTakePictureIntent()
            }

            mDeleteImageButton.setOnClickListener {
                showDeleteImageDialog()
            }
            mActivity.setImageView(view.findViewById(R.id.exerciseImage))
            mActivity.fillImage()
            if(!mIsNew) {
                mSaveButton.text = getString(R.string.save_changes)


                if(mExercise.uid.isNotEmpty()){
                    mActivity.fillImageFromBase64(db.readImage(mExercise.uid))
                }

                if(mExercise.uid.isNotEmpty()){
                    mDeleteImageButton.visibility = View.VISIBLE
                }

                if((mExercise.uid.isEmpty() && mActivity.currentPhotoPath.isEmpty()) || mDeleteImage){
                    mDeleteImageButton.visibility = View.GONE
                    mActivity.clearImage()
                }
            }

            if(mActivity.imageIsEmpty()){
                mDeleteImageButton.visibility = View.GONE
                mActivity.clearImage()
            }


        }

        return view
    }

    override fun onResume() {
        if(((!mIsNew && mExercise.uid.isEmpty()) && mActivity.currentPhotoPath.isNotEmpty() && !mDeleteImage) || (mIsNew && mActivity.currentPhotoPath.isNotEmpty() && !mDeleteImage)){
            val myBitmap = BitmapFactory.decodeFile(mActivity.currentPhotoPath)
            if(myBitmap != null) {
                mDeleteImageButton.visibility = View.VISIBLE
            }
        }
        super.onResume()
    }

    private fun showDeleteImageDialog(){
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(mContext)
        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.yes, null)
        alertDialogBuilder.setCancelable(true).setNegativeButton(R.string.no, null)

        val dialog = alertDialogBuilder.create()
        dialog.setTitle(getString(R.string.delete_image))
        dialog.setMessage(getString(R.string.delete_image_prompt) )

        dialog.setOnShowListener{
            val buttonYes = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
            buttonYes.setOnClickListener{
                mActivity.clearImage()
                if(!mIsNew){
                    if(mExercise.uid.isNotEmpty()) {
                        mDeleteImage = true
                        mActivity.mSharedPreferences.edit().putBoolean("deleteImage", true).apply()
                    }
                }
                mDeleteImageButton.visibility = View.GONE
                dialog.dismiss()
            }

            val buttonNo: Button = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)
            buttonNo.setOnClickListener{
                dialog.dismiss()
            }
        }

        dialog.show()
    }
}