package com.acemondstudios.rootine.UI.Fragments

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Adapters.ExercicesAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Category
import com.acemondstudios.rootine.Models.Day
import com.acemondstudios.rootine.Models.Exercise
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.google.android.material.floatingactionbutton.FloatingActionButton
import java.io.Serializable

class ExerciseListFragment : Fragment(), Serializable {

    private lateinit var mActivity: ContainerActivity
    private lateinit var recyclerView: RecyclerView
    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: LinearLayoutManager

    private var myDataset: ArrayList<Exercise> = ArrayList()

    private lateinit var mDay: Day

    private lateinit var no_data: TextView
    private lateinit var list_icon: ImageView

    private lateinit var newExercise: FloatingActionButton
    private lateinit var playRoutineButton: FloatingActionButton

    private lateinit var db: DBEngine
    private lateinit var mContext: Context

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        //addMockedExercises()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val view: View =  inflater.inflate(R.layout.fragment_exercise_list, container, false)
        mActivity = activity as ContainerActivity

        mContext = context as Context
        db = DBEngine(mContext)

        if(arguments != null) {
            val bundle = arguments as Bundle
            mDay = bundle.get("DAY") as Day

            mActivity.setActionBarTitle(mDay.description)

            viewManager = LinearLayoutManager(context)
            viewAdapter = ExercicesAdapter(context!!, myDataset, this, mActivity, db)

            no_data = view.findViewById(R.id.no_data)
            list_icon = view.findViewById(R.id.list_icon)
            newExercise = view.findViewById(R.id.newExercise)
            playRoutineButton = view.findViewById(R.id.playRoutine)

            recyclerView = view.findViewById(R.id.exercicesRecyclerView)

            recyclerView.apply {
                // use this setting to improve performance if you know that changes
                // in content do not change the layout size of the RecyclerView
                setHasFixedSize(true)

                // use a linear layout manager
                layoutManager = viewManager

                // specify an viewAdapter (see also next example)
                adapter = viewAdapter
            }

            val dividerItemDecoration = DividerItemDecoration(
                recyclerView.context,
                viewManager.orientation
            )
            recyclerView.addItemDecoration(dividerItemDecoration)

            newExercise.setOnClickListener {
                val fragment = ExerciseFragment()
                val args = Bundle()
                args.putSerializable("NEW", null)
                args.putSerializable("DAY", mDay)

                fragment.arguments = args
                mActivity.openFragment(fragment, true, "ExerciseFragment")
            }

            playRoutineButton.setOnClickListener {
                val intent = Intent(context, ContainerActivity::class.java)
                val fragment = RoutineWizardFragment()

                intent.putExtra("FRAGMENT", fragment)
                intent.putExtra("EXERCISES", myDataset)
                intent.putExtra("TAG", "RoutineWizardFragment")
                mContext.startActivity(intent)
            }

            getExercisesFromDatabase()
        }

        mActivity.supportActionBar?.setDisplayHomeAsUpEnabled(true)

        checkNumberOfItems()

        return view;
    }

    override fun onResume() {
        val containerActivity = activity as ContainerActivity
        val sharedPreferences: SharedPreferences =
            containerActivity.getPreferences(Context.MODE_PRIVATE)
        sharedPreferences.edit().remove("currentPhotoPath").apply()
        sharedPreferences.edit().remove("deleteImage").apply()
        mActivity.currentPhotoPath = ""
        super.onResume()
    }

    fun getExercisesFromDatabase(){
        val db = DBEngine(context!!)
        val exercises = db.getExercises(mDay.day_id)
        myDataset.clear()
        myDataset.addAll(exercises)
        viewAdapter.notifyDataSetChanged()
        checkNumberOfItems()
    }

    fun checkNumberOfItems(){
        if(myDataset.size > 0){
            no_data.visibility = View.GONE
            list_icon.visibility = View.GONE
            playRoutineButton.visibility = View.VISIBLE
        }
        else{
            no_data.visibility = View.VISIBLE
            list_icon.visibility = View.VISIBLE
            playRoutineButton.visibility = View.GONE
        }
    }

    private fun addMockedExercises() {
        val category = Category(1, "Pecho")
        val category2 = Category(2, "Bíceps")
        val category3 = Category(3, "Gemelo")
        val exercise = Exercise(1,1,1, category, 1, "Press banca mancuerna", "Realizar con mancuernas, en banco a 45º.", 4, 10, 12, "")
        val exercise2 = Exercise(2,1,1, category, 2, "Press superior palanca", "Realizar en banco de press.", 4, 10, 10, "")
        val exercise3 = Exercise(3,1,1, category, 3, "Aberturas mancuerna", "Realizar de pie con dos mancuernas. Usar cinturón.", 4, 10, 10, "")

        val exercise4 = Exercise(4,1,1, category2, 4, "Curl alterno", "Realizar con mancuernas, sentado en banco.", 4, 10, 10, "")
        val exercise5 = Exercise(5,1,1, category2, 5, "Curl a la vez", "Realizar con mancuernas, sentado en banco.", 4, 10, 10, "")
        val exercise6 = Exercise(6,1,1, category2, 6, "Concentrado", "Realizar en máquina o en banco (con mancuerna)", 3, 12, 12, "")

        val exercise7 = Exercise(7,1,1, category3, 7, "Máquina burro", "Realizar en máquina, calentar bien primero.", 4, 12, 12, "")
        val exercise8 = Exercise(7,1,1, category3, 8, "En prensa", "Realizar en la prensa, usando las puntas de los pies.", 4, 12, 12, "")

        myDataset.add(exercise)
        myDataset.add(exercise2)
        myDataset.add(exercise3)
        myDataset.add(exercise4)
        myDataset.add(exercise5)
        myDataset.add(exercise6)
        myDataset.add(exercise7)
        myDataset.add(exercise8)
    }
}