package com.acemondstudios.rootine.UI.Fragments


import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Adapters.RoutinesAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Routine
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.Utils.Utils
import com.google.android.material.floatingactionbutton.FloatingActionButton
import kotlin.collections.ArrayList


class RoutineListFragment : Fragment() {

    private lateinit var recyclerView: RecyclerView
    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: LinearLayoutManager
    private var myDataset: ArrayList<Routine> = ArrayList()

    private lateinit var no_data: TextView
    private lateinit var list_icon: ImageView

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_routine_list, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewManager = LinearLayoutManager(context)
        viewAdapter = RoutinesAdapter(context!!, myDataset, this)

        no_data = view.findViewById(R.id.no_data)
        list_icon = view.findViewById(R.id.list_icon)

        recyclerView = view.findViewById(R.id.routinesRecyclerView)

        recyclerView.apply {
            // use this setting to improve performance if you know that changes
            // in content do not change the layout size of the RecyclerView
            setHasFixedSize(true)

            // use a linear layout manager
            layoutManager = viewManager

            // specify an viewAdapter (see also next example)
            adapter = viewAdapter
        }

        val dividerItemDecoration = DividerItemDecoration(
            recyclerView.context,
            viewManager.orientation
        )
        recyclerView.addItemDecoration(dividerItemDecoration)

        val newRoutine:FloatingActionButton = view.findViewById(R.id.newRoutine)

        newRoutine.setOnClickListener{
            showNewRoutineDialog(it.context)
        }

        getRoutinesFromDatabase()
    }

    fun getRoutinesFromDatabase(){
        val db = DBEngine(context!!)
        val routines: ArrayList<Routine> = db.getAllRoutines()
        myDataset.clear()
        myDataset.addAll(routines)
        viewAdapter.notifyDataSetChanged()
        checkNumberOfItems()
    }

    fun showNewRoutineDialog(context: Context){
        val layoutInflater = LayoutInflater.from(context)
        val promptView: View =
            layoutInflater.inflate(R.layout.generic_edit_input_dialog, null)
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(context)
        alertDialogBuilder.setView(promptView)

        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.accept, null)

        val dialog = alertDialogBuilder.create()
        dialog.setTitle(context!!.getString(R.string.new_routine))

        val name: EditText = promptView.findViewById(R.id.inputName)
        name.hint = getString(R.string.routine_name)
        name.requestFocus()
        Utils.showKeyboard(context)

        dialog.setOnShowListener{
            val button = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
            button.setOnClickListener{
                if(name.text.isNotEmpty()) {
                    val db = DBEngine(context)
                    db.addRoutine(name.text.toString())
                    dialog.dismiss()
                    getRoutinesFromDatabase()
                }
                else{
                    name.error = it.context.getString(R.string.required_field)
                }
            }
        }

        dialog.setOnDismissListener{
            Utils.closeKeyboard(context)
        }

        dialog.show()
    }

    fun checkNumberOfItems(){
        if(myDataset.size > 0){
            no_data.visibility = View.GONE
            list_icon.visibility = View.GONE
        }
        else{
            no_data.visibility = View.VISIBLE
            list_icon.visibility = View.VISIBLE
        }
    }
}
