package com.acemondstudios.rootine.UI.Fragments

import android.content.Context
import android.graphics.BitmapFactory
import android.os.Bundle
import android.util.Base64
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.Exercise
import com.acemondstudios.rootine.Models.Register
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import java.io.Serializable

class RoutineWizardFragment : Fragment(), Serializable {
    private lateinit var mContainerActivity: ContainerActivity
    private lateinit var mContext: Context
    private lateinit var db: DBEngine
    private lateinit var mExercise: Exercise

    private lateinit var mExerciseTitle: TextView
    private lateinit var mExerciseSeries: TextView
    private lateinit var mExerciseDescription: TextView
    private lateinit var mLastWeight: TextView
    private lateinit var mLastSeries: TextView
    private lateinit var mUsedWeight: EditText
    private lateinit var mSeries: EditText
    private lateinit var mMin: EditText
    private lateinit var mMax: EditText
    private lateinit var mCategoryName: TextView
    private lateinit var mExerciseImage: ImageView

    private lateinit var mButtonNext: Button
    private lateinit var mButtonPrevious: Button
    private lateinit var mButtonFinish: Button
    private lateinit var mButtonCancel: Button

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val v = inflater.inflate(R.layout.fragment_routine_wizard, container, false)
        mContainerActivity = activity as ContainerActivity

        mContext = context as Context
        db = DBEngine(mContext)



        if(arguments != null) {
            val bundle = arguments as Bundle
            mExercise = bundle.getSerializable("CURRENT_EXERCISE") as Exercise

            mExerciseTitle = v.findViewById(R.id.exerciseTitle)
            mExerciseSeries = v.findViewById(R.id.exerciseSeries)
            mExerciseDescription = v.findViewById(R.id.exerciseDescription)
            mLastWeight = v.findViewById(R.id.lastWeight)
            mLastSeries = v.findViewById(R.id.lastSeries)
            mUsedWeight = v.findViewById(R.id.usedWeight)
            mSeries = v.findViewById(R.id.series)
            mMin = v.findViewById(R.id.min)
            mMax = v.findViewById(R.id.max)
            mCategoryName = v.findViewById(R.id.categoryName)
            mExerciseImage = v.findViewById(R.id.exerciseImage)

            mButtonNext = v.findViewById(R.id.nextButton)
            mButtonPrevious = v.findViewById(R.id.previousButton)
            mButtonFinish = v.findViewById(R.id.finishButton)
            mButtonCancel = v.findViewById(R.id.cancelButton)

            mExerciseTitle.text = mExercise.title
            mExerciseDescription.text = mExercise.description
            mExerciseSeries.text = mExercise.series.toString() + " x " + mExercise.min
            mCategoryName.text = mExercise.category.description


            if(mExercise.min != mExercise.max){
                mExerciseSeries.text = mExerciseSeries.text as String + "-" + mExercise.max
            }

            val lastWeight = db.getLastWeight(mExercise.exercise_id, mExercise.day_id, mExercise.routine_id)
            val lastSeries = db.getLastSeries(mExercise.exercise_id, mExercise.day_id, mExercise.routine_id)

            if(lastWeight > 0){
                mLastWeight.setText(getText(R.string.last_weight).toString() + " " + lastWeight.toString().replace(".", ",") + " kg")
            }
            else {
                mLastWeight.visibility = View.GONE
            }

            if(lastSeries.isNotEmpty()){
                mLastSeries.setText(getText(R.string.last_series).toString() + " " + lastSeries)
            }
            else{
                mLastSeries.visibility = View.GONE
            }

            mButtonNext.setOnClickListener {
                if(this.validateForm()) {
                    mContainerActivity.nextButton(
                        mUsedWeight.text.toString().toDouble(),
                        mSeries.text.toString().toInt(),
                        mMin.text.toString().toInt(),
                        mMax.text.toString().toInt()
                    )
                }
            }

            mButtonPrevious.setOnClickListener {
                var usedWeight = 0.0
                var series = 0
                var min = 0
                var max = 0
                if(mUsedWeight.text.isNotEmpty()){
                    usedWeight = mUsedWeight.text.toString().toDouble()
                }
                if(mSeries.text.isNotEmpty()){
                    series = mSeries.text.toString().toInt()
                }
                if(mMin.text.isNotEmpty()){
                    min =  mMin.text.toString().toInt()
                }
                if(mMax.text.isNotEmpty()){
                    max = mMax.text.toString().toInt()
                }

                mContainerActivity.previousButton(
                   usedWeight,
                    series,
                    min,
                    max
                )
            }

            mButtonFinish.setOnClickListener {
                mContainerActivity.finishButton(
                    mUsedWeight.text.toString().toDouble(),
                    mSeries.text.toString().toInt(),
                    mMin.text.toString().toInt(),
                    mMax.text.toString().toInt())
            }

            mButtonCancel.setOnClickListener {
                mContainerActivity.cancelWizard()
            }

            if(mContainerActivity.isLastExercise()){
                mButtonNext.visibility = View.GONE
                mButtonFinish.visibility = View.VISIBLE
            }

            if(mContainerActivity.isFirstExercise()){
                mButtonPrevious.visibility = View.GONE
                mButtonCancel.visibility = View.VISIBLE
            }
            else{
                mButtonCancel.visibility = View.GONE
            }

            if(bundle.containsKey("CURRENT_REGISTER")){
                val register = bundle.getSerializable("CURRENT_REGISTER") as Register
                if(register.exercise_id == mExercise.exercise_id
                    && register.routine_id == mExercise.routine_id
                    && register.day_id == mExercise.day_id) {
                    if (register.weight > 0)
                        mUsedWeight.setText(register.weight.toString())
                    if (register.series > 0)
                        mSeries.setText(register.series.toString())
                    if (register.min > 0)
                        mMin.setText(register.min.toString())
                    if (register.max > 0)
                        mMax.setText(register.max.toString())
                }
            }

            if(mExercise.uid.isNotEmpty()){
                val base64 = db.readImage(mExercise.uid)
                val decodedString: ByteArray = Base64.decode(base64, Base64.DEFAULT)
                val decodedByte = BitmapFactory.decodeByteArray(decodedString, 0, decodedString.size)
                if(decodedByte != null) {
                    mExerciseImage.setImageBitmap(decodedByte)
                    mExerciseImage.scaleType = ImageView.ScaleType.CENTER_CROP
                }
            }
        }
        return v
    }

    fun validateForm(): Boolean{
        var isValid = true
        if(!mUsedWeight.text.isNotEmpty()){
            isValid = false
            mUsedWeight.error = mContainerActivity.getString(R.string.required_field)
        }
        if(!mSeries.text.isNotEmpty()){
            isValid = false
            mSeries.error = mContainerActivity.getString(R.string.required_field)
        }
        if(!mMin.text.isNotEmpty()){
            isValid = false
            mMin.error = mContainerActivity.getString(R.string.required_field)
        }
        if(!mMax.text.isNotEmpty()){
            isValid = false
            mMax.error = mContainerActivity.getString(R.string.required_field)
        }
        return isValid
    }

}