package com.acemondstudios.rootine.UI.Fragments


import android.content.Context
import android.os.Bundle
import android.text.method.LinkMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import androidx.core.text.HtmlCompat
import androidx.fragment.app.Fragment
import com.acemondstudios.rootine.Adapters.ExerciseSpinnerAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.ExerciseSpinnerModel
import com.acemondstudios.rootine.Models.UserWeight
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.Utils.Utils
import com.github.mikephil.charting.charts.LineChart
import com.github.mikephil.charting.components.AxisBase
import com.github.mikephil.charting.components.Description
import com.github.mikephil.charting.components.XAxis
import com.github.mikephil.charting.components.YAxis
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.github.mikephil.charting.formatter.ValueFormatter
import org.w3c.dom.Text


/**
 * A simple [Fragment] subclass.
 */
class StatisticsFragment : Fragment() {

    private lateinit var mContext: Context
    private lateinit var mUserWeightList: ArrayList<UserWeight>
    private lateinit var mExerciseWeightList: ArrayList<UserWeight>
    private lateinit var mExerciseSpinnerModels: ArrayList<ExerciseSpinnerModel>
    private lateinit var mUserWeightChart: LineChart
    private lateinit var mExerciseWeightChart: LineChart
    private lateinit var mWeightHistoryEmpty: TextView
    private lateinit var mWeightPerExerciseEmpty: TextView
    private lateinit var mAboutApp: ImageView
    private lateinit var mDb: DBEngine

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_statistics, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        mContext = context as Context
        mExerciseWeightChart = view.findViewById(R.id.weightPerExereciseChart) as LineChart
        mUserWeightChart = view.findViewById(R.id.weightChart) as LineChart
        mWeightHistoryEmpty = view.findViewById(R.id.weightHistoryEmpty) as TextView
        mWeightPerExerciseEmpty = view.findViewById(R.id.weightPerExerciseEmpty) as TextView
        mAboutApp = view.findViewById(R.id.aboutApp) as ImageView

        mAboutApp.setOnClickListener {
            openLicensePopup()
        }

        mDb = DBEngine(mContext)
        //mUserWeightList = Utils.getMockedWeightData(20)
        mUserWeightList = mDb.getAllUserWeight()
        mExerciseWeightList = ArrayList()
        //mUserWeightList = ArrayList<UserWeight>()
        mExerciseSpinnerModels = mDb.getExerciseSpinnerModels()
        val exercisesSpinner = view.findViewById(R.id.exercisesSpinner) as Spinner

        if (mExerciseSpinnerModels.size == 0) {
            exercisesSpinner.visibility = View.GONE
            mExerciseWeightChart.visibility = View.GONE
            mWeightPerExerciseEmpty.visibility = View.VISIBLE
        } else {
            exercisesSpinner.adapter = ExerciseSpinnerAdapter(
                mContext,
                android.R.layout.simple_spinner_dropdown_item,
                mExerciseSpinnerModels
            )
            exercisesSpinner.setSelection(0)

            exercisesSpinner.onItemSelectedListener =
                object : AdapterView.OnItemSelectedListener {
                    override fun onNothingSelected(parent: AdapterView<*>?) {

                    }

                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        mExerciseWeightList = mDb.getAllExerciseWeightRecords(mExerciseSpinnerModels[position].exercise_id)
                        if (mExerciseWeightList.size > 0) {
                            mExerciseWeightChart.visibility = View.VISIBLE
                            mWeightPerExerciseEmpty.visibility = View.GONE
                            mExerciseWeightChart.clear()
                            prepareExerciseChart()
                        } else {
                            mExerciseWeightChart.visibility = View.GONE
                            mWeightPerExerciseEmpty.visibility = View.VISIBLE
                        }
                    }
                }
        }

        prepareWeightChart()
    }

    private fun prepareWeightChart(){
        prepareChart(mUserWeightChart, mUserWeightList, mWeightHistoryEmpty, getString(R.string.insufficient_weight_registers))
    }

    private fun prepareExerciseChart(){
        prepareChart(mExerciseWeightChart, mExerciseWeightList, mWeightPerExerciseEmpty, getString(R.string.insufficient_registers))
    }

    private fun prepareChart(chart: LineChart, weightList: ArrayList<UserWeight>, emptyMessage: TextView, insufficientRegisters: String){
        val quarters = ArrayList<String>()
        val entries = ArrayList<Entry>()

        var i = 0.0F
        for(exerciseWeight in weightList) {
            val currentDate = Utils.getFormattedDate(exerciseWeight.record_date).split(' ')[0]
            quarters.add(Utils.getShortedDate(currentDate))
            entries.add(Entry(i, exerciseWeight.weight.toFloat()))
            i += 1.0F
        }

        if(weightList.size < 2){
            chart.visibility = View.GONE
            emptyMessage.visibility = View.VISIBLE
            if(weightList.size == 1)
                emptyMessage.text = insufficientRegisters
        }
        else {
            drawChart(chart, quarters, entries, weightList)
        }
    }

    private fun drawChart(chart: LineChart, quarters: ArrayList<String>, entries: ArrayList<Entry>, weightList: ArrayList<UserWeight>){
        val formatter: ValueFormatter = object : ValueFormatter() {
            override fun getAxisLabel(value: Float, axis: AxisBase): String {
                val index = value.toInt()
                return if(index < quarters.size)
                    quarters[value.toInt()]
                else ""
            }
        }

        val xAxis: XAxis = chart.xAxis

        xAxis.granularity = 1f
        xAxis.labelCount = quarters.size

        xAxis.yOffset = 5.0F
        xAxis.textColor = ContextCompat.getColor(mContext, R.color.chartText)

        if(weightList.size > 5){
            xAxis.labelRotationAngle = 45.0F
            chart.extraRightOffset = 20.0F
        }
        else {
            chart.extraRightOffset = 20.0F
        }

        chart.extraLeftOffset = 5.0F

        xAxis.valueFormatter = formatter

        val axisLeft: YAxis = chart.axisLeft
        axisLeft.xOffset = 6.0F
        axisLeft.textColor = ContextCompat.getColor(mContext, R.color.chartText)

        val axisRight: YAxis = chart.axisRight
        axisRight.isEnabled = false

        val dataSet = LineDataSet(entries, getString(R.string.weight))
        // add entries to dataset
        dataSet.color = ContextCompat.getColor(mContext, R.color.primary)
        dataSet.setCircleColor(ContextCompat.getColor(mContext, R.color.primary))
        dataSet.circleRadius = 4.0F
        dataSet.lineWidth = 2.0F
        dataSet.valueTextColor = ContextCompat.getColor(mContext, R.color.chartText)

        val lineData = LineData(dataSet)
        chart.data = lineData

        chart.setBackgroundColor(ContextCompat.getColor(mContext, R.color.chartBackground))
        val chartDescription = Description()
        chartDescription.text = ""
        chart.isClickable = false
        chart.description = chartDescription
        chart.setNoDataText(getString(R.string.no_info))
        chart.setNoDataTextColor(ContextCompat.getColor(mContext, R.color.chartText))
        chart.legend.isEnabled = false
        chart.setScaleEnabled(false)
        chart.setTouchEnabled(false)
        chart.setDrawBorders(true)
        chart.setBorderColor(ContextCompat.getColor(mContext, R.color.chartText))
        chart.invalidate()
    }

    private fun openLicensePopup(){
        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(mContext)
        // setup a dialog window
        alertDialogBuilder.setCancelable(true).setPositiveButton(R.string.ok, null)

        val dialog: AlertDialog = alertDialogBuilder.create()
        dialog.setTitle(getString(R.string.about))
        dialog.setMessage(HtmlCompat.fromHtml(getString(R.string.license), HtmlCompat.FROM_HTML_MODE_LEGACY))

        dialog.show()
    }

}
