package com.acemondstudios.rootine.UI.Fragments


import android.R.attr.data
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Adapters.RoutineHistoryRegisterAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.RoutineHistoryRegister
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import com.acemondstudios.rootine.UI.Activities.TabActivity


class UserFragment : Fragment() {
    private lateinit var mActivity: TabActivity
    private lateinit var mContext: Context
    private lateinit var mAddWeightButton: Button
    private lateinit var mWeightHistoryButton: Button
    private lateinit var mCurrentWeight: TextView
    private lateinit var mNoData: TextView
    private lateinit var mRoutineHistoryRecyclerView: RecyclerView
    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: LinearLayoutManager

    private var mRoutineHistoryRegisterList: ArrayList<RoutineHistoryRegister> = ArrayList()
    private lateinit var db: DBEngine

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        val view =  inflater.inflate(R.layout.fragment_user, container, false)
        mActivity = activity as TabActivity
        mContext = context as Context

        db = DBEngine(mContext)

        mAddWeightButton = view.findViewById(R.id.addWeightButton)
        mCurrentWeight = view.findViewById(R.id.weightAdd)
        mWeightHistoryButton = view.findViewById(R.id.weightHistoryButton)

        mAddWeightButton.setOnClickListener {
            if(mCurrentWeight.text.toString().isEmpty() || mCurrentWeight.text.toString().toDouble() == 0.0){
                mCurrentWeight.error = getString(R.string.required_field)
            }
            else{
                // Everything OK, it's possible to add weight registry
                db.createUserWeight(mCurrentWeight.text.toString().toDouble())
                mCurrentWeight.text = ""
                Toast.makeText(mContext, R.string.weight_add_ok, Toast.LENGTH_LONG).show()
            }

        }

        mWeightHistoryButton.setOnClickListener {
            val intent = Intent(mContext, ContainerActivity::class.java)
            val fragment = WeightHistoryFragment()

            intent.putExtra("FRAGMENT", fragment)
            intent.putExtra("TAG", "WeightHistoryFragment")
            mContext.startActivity(intent)
        }
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewManager = LinearLayoutManager(context)
        viewAdapter = RoutineHistoryRegisterAdapter(context!!, mRoutineHistoryRegisterList, this, db)

        mRoutineHistoryRecyclerView = view.findViewById(R.id.routineHistoryRecyclerView)

        mRoutineHistoryRecyclerView.isNestedScrollingEnabled = false;

        mRoutineHistoryRecyclerView.apply {
            // use this setting to improve performance if you know that changes
            // in content do not change the layout size of the RecyclerView
            setHasFixedSize(true)

            // use a linear layout manager
            layoutManager = viewManager

            // specify an viewAdapter (see also next example)
            adapter = viewAdapter
        }

        val dividerItemDecoration = DividerItemDecoration(
            mRoutineHistoryRecyclerView.context,
            viewManager.orientation
        )
        mRoutineHistoryRecyclerView.addItemDecoration(dividerItemDecoration)

        mNoData = view.findViewById(R.id.no_data)
        getRoutineHistoryRegistersFromDatabase()
    }

    override fun onResume() {
        super.onResume()
        getRoutineHistoryRegistersFromDatabase()
    }

    fun getRoutineHistoryRegistersFromDatabase(){
        val db = DBEngine(context!!)
        val routines: ArrayList<RoutineHistoryRegister> = db.getRoutineHistory()
        this.clear()
        mRoutineHistoryRegisterList.addAll(routines)
        viewAdapter.notifyDataSetChanged()
        checkNumberOfItems()
    }

    fun clear() {
        val size: Int = mRoutineHistoryRegisterList.size
        if (size > 0) {
            for (i in 0 until size) {
                mRoutineHistoryRegisterList.removeAt(0)
            }
            viewAdapter.notifyItemRangeRemoved(0, size)
        }
    }

    private fun checkNumberOfItems(){
        if(mRoutineHistoryRegisterList.count() > 0){
            mRoutineHistoryRecyclerView.visibility = View.VISIBLE
            mNoData.visibility = View.GONE
        }
        else{
            mRoutineHistoryRecyclerView.visibility = View.GONE
            mNoData.visibility = View.VISIBLE
        }
    }


}
