package com.acemondstudios.rootine.UI.Fragments

import android.content.Context
import android.os.Bundle
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.acemondstudios.rootine.Adapters.UserWeightAdapter
import com.acemondstudios.rootine.Database.DBEngine
import com.acemondstudios.rootine.Models.UserWeight
import com.acemondstudios.rootine.R
import com.acemondstudios.rootine.UI.Activities.ContainerActivity
import java.io.Serializable

class WeightHistoryFragment : Fragment(), Serializable {

    private lateinit var mContainerActivity: ContainerActivity
    private lateinit var mListIcon: ImageView
    private lateinit var mContext: Context
    private lateinit var mDBEngine: DBEngine
    private lateinit var mRecyclerView: RecyclerView
    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: LinearLayoutManager
    private lateinit var mInformation: TextView
    private var mUserWeightList: ArrayList<UserWeight> = ArrayList()


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_weight_history, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        mContainerActivity = activity as ContainerActivity
        mContext = context as Context
        mDBEngine = DBEngine(mContext)
        mContainerActivity.setActionBarTitle(getString(R.string.history_title))
        mContainerActivity.supportActionBar?.setDisplayHomeAsUpEnabled(true)

        mListIcon = view.findViewById(R.id.list_icon)
        mRecyclerView = view.findViewById(R.id.weightRecyclerView)
        mInformation = view.findViewById(R.id.weightInformation)

        viewManager = LinearLayoutManager(context)
        viewAdapter = UserWeightAdapter(context!!, mUserWeightList, this, mContainerActivity)

        mRecyclerView.apply {
            // use this setting to improve performance if you know that changes
            // in content do not change the layout size of the RecyclerView
            setHasFixedSize(true)

            // use a linear layout manager
            layoutManager = viewManager

            // specify an viewAdapter (see also next example)
            adapter = viewAdapter
        }

        val dividerItemDecoration = DividerItemDecoration(
            mRecyclerView.context,
            viewManager.orientation
        )
        mRecyclerView.addItemDecoration(dividerItemDecoration)

        getUserWeightsFromDatabase()
    }

    fun getUserWeightsFromDatabase(){
        val userWeights: ArrayList<UserWeight> = mDBEngine.getAllUserWeight()
        mUserWeightList.clear()
        mUserWeightList.addAll(userWeights)
        viewAdapter.notifyDataSetChanged()
        checkNumberOfItems()
    }

    private fun checkNumberOfItems(){
        if(mUserWeightList.size > 0){
            mInformation.visibility = View.GONE
            mRecyclerView.visibility = View.VISIBLE
            mListIcon.visibility = View.GONE
        }
        else{
            mInformation.visibility = View.VISIBLE
            mRecyclerView.visibility = View.GONE
            mListIcon.visibility = View.VISIBLE
        }
    }
}