package com.acemondstudios.rootine.Utils

import android.content.Context
import android.graphics.Bitmap
import android.view.inputmethod.InputMethodManager
import com.acemondstudios.rootine.Models.UserWeight
import java.util.*
import kotlin.collections.ArrayList


object Utils {
    @JvmStatic
    fun getFormattedDate(calendar: Calendar) : String {
        val day = calendar.get(Calendar.DAY_OF_MONTH)
        val month = (calendar.get(Calendar.MONTH) + 1)
        val year = calendar.get(Calendar.YEAR).toString()
        val hours = calendar.get(Calendar.HOUR_OF_DAY)
        val minutes = calendar.get(Calendar.MINUTE)
        val seconds = calendar.get(Calendar.SECOND)

        val strDay = formatDateNumber(day)
        val strMonth = formatDateNumber(month)
        val strHours = formatDateNumber(hours)
        val strMinutes = formatDateNumber(minutes)
        val strSeconds = formatDateNumber(seconds)

        return "$strDay/$strMonth/$year $strHours:$strMinutes:$strSeconds"
    }

    private fun formatDateNumber(num: Int): String{
        if(num < 10){
            return "0$num"
        }
        return num.toString()
    }

    fun getSimpleFormattedDate(calendar: Calendar) : String {
        val day = calendar.get(Calendar.DAY_OF_MONTH)
        val month = calendar.get(Calendar.MONTH) + 1
        val year = calendar.get(Calendar.YEAR)

        var dayStr = day.toString()
        if(day < 10){
            dayStr = "0$dayStr"
        }

        var monthStr = month.toString()
        if(month < 10){
            monthStr = "0$monthStr"
        }

        return "$dayStr/$monthStr/$year"
    }

    fun getCurrentStringDate() : String{
        return getFormattedDate(Calendar.getInstance())
    }

    fun getStringDate(calendar: Calendar): String{
        return getFormattedDate(calendar)
    }

    fun showKeyboard(context: Context) {
        val inputMethodManager: InputMethodManager =
            context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.toggleSoftInput(InputMethodManager.SHOW_FORCED, 0)
    }

    fun closeKeyboard(context: Context) {
        val inputMethodManager =
            context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.toggleSoftInput(
            InputMethodManager.HIDE_IMPLICIT_ONLY,
            0
        )
    }

    fun getResizedBitmap(image: Bitmap, maxSize: Int): Bitmap? {
        var width = image.width
        var height = image.height
        val bitmapRatio = width.toFloat() / height.toFloat()
        if (bitmapRatio > 1) {
            width = maxSize
            height = (width / bitmapRatio).toInt()
        } else {
            height = maxSize
            width = (height * bitmapRatio).toInt()
        }
        return Bitmap.createScaledBitmap(image, width, height, true)
    }

    fun getShortedDate(date: String): String{
        var res = date.split('/')
        return res[0] + "/" + res[1] + "/" + res[2].substring(2)
    }

    fun getMockedWeightData(numberOfValues: Int): ArrayList<UserWeight> {
        val list = ArrayList<UserWeight>()

        for(i in 1..numberOfValues){
            val currDate = Calendar.getInstance()
            currDate.set(Calendar.DAY_OF_MONTH, rand(1, 28))
            currDate.set(Calendar.MONTH, rand(0, 11))
            currDate.set(Calendar.YEAR, 2021)
            val curr = UserWeight(currDate, this.rand(70, 120).toDouble())
            list.add(curr)
        }

        return list
    }

    fun rand(start: Int, end: Int): Int {
        require(!(start > end || end - start + 1 > Int.MAX_VALUE)) { "Illegal Argument" }
        return Random(System.nanoTime()).nextInt(end - start + 1) + start
    }
}